/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <pspkernel.h>
#include <psppower.h>
#include <pspctrl.h>
#include <pspwlan.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <SDL.h>

#include "psp_global.h"
#include "psp_login.h"
#include "psp_sdl.h"
#include "psp_console.h"
#include "irc_main.h"
#include "irc_global.h"
#include "my_socket.h"

//////////////////////////////////////////////////////////////////
//Bits for resolver
int  m_ResolverId;
char m_ResolverBuffer[1024]; /** Could be smaller, no idea */

/* messageBox for the UI Thread to message the Network Thread */
SceUID networkMessagebox;
/* messageBox for the networkProtocol thread (Telnet / IRC / SSH) */
SceUID protocolMessagebox;


void
irc_main_init()
{
  if (! networkMessagebox) {
	  //Prepare Mailboxes for communication between threads
	  networkMessagebox = sceKernelCreateMbx("NT-MB", 0, 0);
    //renderMessagebox = sceKernelCreateMbx("UI-MB", 0, 0);
	  protocolMessagebox = sceKernelCreateMbx("PT-MB", 0, 0);
  }
}

void
irc_main_render(char *buffer, int color)
{
  psp_console_write(buffer, color);
}

struct sockaddr_in 
stringTosockaddr(char* input)
{
  struct sockaddr_in addrListen;
  
  addrListen.sin_family = AF_INET;
  addrListen.sin_addr[0] = 0;
  addrListen.sin_addr[1] = 0;
  addrListen.sin_addr[2] = 0;
  addrListen.sin_addr[3] = 0;
  addrListen.sin_port = 0;

  int foundcolon = 0;
  int colonpos = 0;
  for (;colonpos < strlen(input); colonpos++)
  {
    if (input[colonpos] == ':')
    {
      foundcolon = 1;
      break;
    }
  }
  if (!foundcolon) {
    return addrListen;
  }
  
  //temporarily make colonpos = '\0'
  input[colonpos] = '\0';
  
  struct in_addr addr;

  /* RC Let's try aton first in case the address is in dotted numerical form */
  memset(&addr, 0, sizeof(struct in_addr));
  int rc = sceNetInetInetAton(input, &addr);
  if (rc == 0)
  {
    /* That didn't work!, it must be a hostname, let's try the resolver... */
    irc_main_render("Resolving Host\n", COLOR_WHITE);
    rc = sceNetResolverStartNtoA(m_ResolverId, input, &addr, 2, 3);
    if (rc < 0)
    {
      irc_main_render("Could not resolve host!\n", COLOR_RED);
      return addrListen;
    }
  }
  
  memcpy(&addrListen.sin_addr, &addr, sizeof(struct in_addr));
  
  input[colonpos] = ':';
  //now parse out the port after the :
  colonpos++;
  while( input[colonpos] >= '0' && input[colonpos] <= '9')
  {
    addrListen.sin_port *= 10;
    addrListen.sin_port += input[colonpos] - '0';
    colonpos++;
  }
  
  addrListen.sin_port = htons(addrListen.sin_port);
  
  return addrListen;
}

extern int ircProtocol(SceSize args, void *argp);

static SOCKET sockListen = -1;
static volatile int loc_session_terminated = 1;

int 
irc_main_open_session()
{
  u32 err;
  char buffer[200];
  char server_name[200];

  psp_console_clear_screen();

  //Create a Socket
  struct sockaddr_in addrListen;
  loc_session_terminated = 1;

  sockListen = sceNetInetSocket(AF_INET, SOCK_STREAM, 0);
  if (sockListen <= 0) {
    sprintf(buffer, "socket returned $%x\n", sockListen);
    irc_main_render(buffer, COLOR_RED);
    return -1;
  }
  loc_session_terminated = 0;

  int rc = sceNetResolverCreate(&m_ResolverId, m_ResolverBuffer, sizeof(m_ResolverBuffer));
  if (rc < 0) {
    sprintf(buffer, "sceNetResolverCreate $%x\n", rc);
    irc_main_render(buffer, COLOR_RED);
    irc_main_close_session();
    return -1;
  }
    
  sprintf(server_name, "%s:%s", psp_login_current->login.host, psp_login_current->login.port);
  //Get CON IP From User
  addrListen = stringTosockaddr(server_name);
  if (addrListen.sin_port == 0) { //user gave invalid details
    irc_main_close_session();
    return -1;
  }
      
  sprintf(buffer, "Connecting to: %s\n", server_name);
  irc_main_render(buffer, COLOR_WHITE);
  sprintf(buffer, "IP %d.%d.%d.%d:%d\n", 
     addrListen.sin_addr[0], addrListen.sin_addr[1], 
     addrListen.sin_addr[2], addrListen.sin_addr[3], htons(addrListen.sin_port));
  irc_main_render(buffer, COLOR_WHITE);
    
  err = sceNetInetConnect(sockListen, &addrListen, sizeof(addrListen));
  if (err != 0) {
    irc_main_render("Unable to connect!\n", COLOR_RED);
      
    switch(sceNetInetGetErrno())
    {
      case 0x74:
        irc_main_render("  Connection Timed out\n", COLOR_RED);
        break;
      case 0x6f:
        irc_main_render("  Connection Refused\n", COLOR_RED);
        break;
      default:
        sprintf(buffer, "connect returned $%x\n", err);
        irc_main_render(buffer, COLOR_RED);
        sprintf(buffer, "  errno=$%x\n", sceNetInetGetErrno());
        irc_main_render(buffer, COLOR_RED);
    }
    irc_main_close_session();
    return -1;
  }
    
  //disable nonblock maybe
  //This bit thanks to XOSs pspvnc :) you guys rock.
#define IPPROTO_TCP     6
#define TCP_NODELAY     0x0001
  u32 one = 1;
  sceNetInetSetsockopt(sockListen, IPPROTO_TCP, TCP_NODELAY,(char *)&one, sizeof(one));
    
  //Alter Socket options to have a timeout
  u32 timeout = 1000000; // in microseconds == 1 sec
  err = sceNetInetSetsockopt(sockListen, SOL_SOCKET, SO_RCVTIMEO, (char*)&timeout, sizeof(timeout));
    
  if (err != 0) //Failure
  {
    irc_main_render("set SO_RCVTIMEO failed\n", COLOR_RED);
    u32 data;
    int len;
    err = sceNetInetGetsockopt(sockListen, SOL_SOCKET, SO_RCVTIMEO,
    (char*)&data, &len);
    if (err == 0 && len == 4) {
      sprintf(buffer,"Get SO_RCVTIMEO = %d ($%x)\n", data, data);
      irc_main_render(buffer, COLOR_WHITE);
    }
  }
    
  //We are all Connected, 
  sprintf(buffer,"Successfully connected\n");
  irc_main_render(buffer, COLOR_WHITE);
  return 0;
}

void
irc_main_loop_session()
{
  int err;
  int error;
  irc_main_render("Entering main loop\n", COLOR_WHITE);

  while (! irc_main_session_is_terminated()) {

      sceKernelDelayThread(1000);

      //Deal with Messages in the Box
      NetworkMsg* data;
      error = sceKernelPollMbx(networkMessagebox, (void*)&data);
      if(error < 0) {
        /* Nothing Arrived */
      } else {
        //Received a Message
        //Send it over the Network.
        int j;
        j = sceNetInetSend(sockListen, data->text, data->length, 0);
# if 0 //LUDO:
        if (j < 0) {
          irc_main_render("SEND FAILED!!!!!!!!!\n", COLOR_RED);
        }
# endif
        free(data->text);
        free(data);
      }

      //Receive anything on the Socket
      char buffer[2048];
      int cch;

      cch = sceNetInetRecv(sockListen, (u8*)buffer, sizeof(buffer)-1, 0);
      if (cch == 0)
        break;      // connection closed

      if (cch < 0) {
        int errno = sceNetInetGetErrno();
        if (errno == 11) {
          // recv timeout, have a sleep
          sceKernelDelayThread(1000);
        }
      }
      else
      {
        //successfull receive
        buffer[cch] = '\0';
        protocolSendBin(buffer, cch, FROM_NETWORK);
      }
      
# if 0 //LUDO:
      if (sceWlanDevIsPowerOn() != 1) //power is off
      {
        irc_main_render("WLAN has lost power, psp went to sleep?\n", COLOR_RED);
        break;
      }
# endif
  }
  irc_main_session_set_terminated();
  err = sceNetInetClose(sockListen);
# if 0 //LUDO:
  if (err != 0) {
      char buffer[128];
      sprintf(buffer,"closesocket returned $%x\n", err);
      irc_main_render(buffer, COLOR_RED);
  }
# endif
  irc_main_close_session();
}

int
irc_main_session_is_terminated()
{
  return loc_session_terminated;
}

void
irc_main_session_set_terminated()
{
  loc_session_terminated = 1;
}

static SceUID protocolthid = -1;

void
irc_main_start_proto_thread()
{
  //Launch the protocol thread for that favorite
  protocolthid = sceKernelCreateThread("Protocol Thread", ircProtocol , 0x11, 128*1024, 
                  PSP_THREAD_ATTR_USER, 0);
  if (protocolthid >= 0) {
    sceKernelStartThread(protocolthid, 0, NULL);
  } else {
    psp_sdl_exit(1);
  }
}

void
irc_main_stop_proto_thread()
{
  loc_session_terminated = 1;
# if 0
  if (protocolthid != -1) {
    sceKernelTerminateThread(protocolthid);
    protocolthid = -1;
  }
# endif
  protocolthid = -1;
}

int 
irc_main_close_session()
{
  int err = 0;
  //Try close the socket incase it isnt
  if (sockListen != -1) {
    err = sceNetInetClose(sockListen);
    sockListen = -1;
  }
  loc_session_terminated = 1;
  return err;
}
