/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <pspkernel.h>
#include <psppower.h>
#include <pspctrl.h>
#include <pspwlan.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <SDL.h>

#include "psp_global.h"
#include "psp_sdl.h"
#include "psp_menu.h"
#include "psp_danzeff.h"
#include "psp_fmgr.h"
#include "psp_irc.h"
#include "psp_irkeyb.h"
#include "irc_global.h"

static int loc_irc_thid = -1;

void
psp_irc_display_screen()
{
  psp_console_display_screen();
}

void
psp_irc_thread(SceSize argc, void *argv)
{
  irc_main_loop_session();
}

void
psp_irc_clear_screen()
{
  psp_irc_display_screen();
}

void
psp_irc_start_main_thread()
{
  /* Launch irc client thread */
  loc_irc_thid = sceKernelCreateThread( "irc_thread", 
     (SceKernelThreadEntry)psp_irc_thread, 0x18, 0x10000, 0, 0 );

  if(loc_irc_thid >= 0) {
    sceKernelStartThread(loc_irc_thid, 0, 0);
  } else {
    psp_sdl_exit(1);
  }
}

void
psp_irc_stop_main_thread()
{
# if 0 //LUDO:
  if (loc_irc_thid != -1) {
    sceKernelExitDeleteThread(loc_irc_thid);
    sceKernelWaitThreadEnd(loc_irc_thid, NULL);
    loc_irc_thid = -1;
  }
# endif
  loc_irc_thid = -1;
}

void
psp_irc_main()
{
  SceCtrlData c;

  long new_pad;
  long old_pad;

  int  danzeff_mode = 0;
  int  danzeff_key;
# ifdef USE_PSP_IRKEYB
  int  irkeyb_mode = 0;
  int  irkeyb_key = 0;
# endif

  int last_time  = 0;

  protocolSend("hello", FROM_CONFIG);

  while (! irc_main_session_is_terminated())  {

    sceKernelDelayThread(1000);

    psp_irc_display_screen();

    if (danzeff_mode) {
      danzeff_moveTo(-50, -50);
      danzeff_render();
    }
    psp_sdl_flip();

    if (PSPIRC.psp_screenshot_mode) {
      PSPIRC.psp_screenshot_mode--;
      if (PSPIRC.psp_screenshot_mode <= 0) {
        psp_sdl_save_screenshot();
        PSPIRC.psp_screenshot_mode = 0;
      }
    }
    if (PSPIRC.psp_save_log_mode) {
      PSPIRC.psp_save_log_mode--;
      if (PSPIRC.psp_save_log_mode <= 0) {
        psp_console_save_log();
        PSPIRC.psp_save_log_mode = 0;
      }
    }

# ifdef USE_PSP_IRKEYB
    irkeyb_mode = 0;
# endif
    sceDisplayWaitVblankStart();
 
    while (1) {

      sceCtrlPeekBufferPositive(&c, 1);
      c.Buttons &= PSP_ALL_BUTTON_MASK;

# ifdef USE_PSP_IRKEYB
      irkeyb_key = psp_irkeyb_set_psp_key(&c);
# endif
      new_pad = c.Buttons;

      if ((old_pad != new_pad) || ((c.TimeStamp - last_time) > PSP_MENU_MIN_TIME)) {
        last_time = c.TimeStamp;
        old_pad = new_pad;
        break;
      }

# ifdef USE_PSP_IRKEYB
      irkeyb_key = psp_irkeyb_read_key();
      if (irkeyb_key != PSP_IRKEYB_EMPTY) {
        if (irkeyb_key >= ' ') {
          psp_console_put_key(irkeyb_key);
        } else 
        if (irkeyb_key == 0x8) {
          psp_console_put_key(DANZEFF_DEL);
        } else
        if (irkeyb_key == 0xd) {
          psp_console_put_key(DANZEFF_ENTER);
        }
        last_time = c.TimeStamp;
        irkeyb_mode = 1;
        break;
      }
# endif
    }
# ifdef USE_PSP_IRKEYB
    if (irkeyb_mode) continue;
# endif

    if (danzeff_mode) {

      danzeff_key = danzeff_readInput(c);

      if (danzeff_key > DANZEFF_START) {
        if (danzeff_key >= ' ') {
          psp_console_put_key(danzeff_key);
        } else
        if ((danzeff_key == DANZEFF_DEL ) ||
            (danzeff_key == DANZEFF_LEFT)) {
          psp_console_put_key(DANZEFF_DEL);
        } else
        if ((danzeff_key == DANZEFF_ENTER) ||
            (danzeff_key == DANZEFF_DOWN )) {
          psp_console_put_key(DANZEFF_ENTER);
        } else
        if (danzeff_key == DANZEFF_UP) {
          psp_console_put_key('/');
        } else
        if (danzeff_key == DANZEFF_RIGHT) {
          psp_console_put_key(' ');
        }

      } else
      if ((danzeff_key == DANZEFF_START ) || 
          (danzeff_key == DANZEFF_SELECT)) 
      {
        danzeff_mode = 0;
        old_pad = new_pad = 0;

        psp_kbd_wait_no_button();
      }

      continue;
    }


    if ((c.Buttons & (PSP_CTRL_LTRIGGER|PSP_CTRL_RTRIGGER|PSP_CTRL_START)) ==
        (PSP_CTRL_LTRIGGER|PSP_CTRL_RTRIGGER|PSP_CTRL_START)) {
      /* Exit ! */
      psp_sdl_exit(0);
    } else
    if(new_pad & PSP_CTRL_SELECT) {
      /* Back to Main Menu */
      psp_irc_menu();

    } else
    if(new_pad & PSP_CTRL_START) {
      danzeff_mode = 1;
    } else
    if(new_pad & PSP_CTRL_LTRIGGER) {
      psp_console_screen_scroll_down();
    } else
    if(new_pad & PSP_CTRL_RTRIGGER) {
      psp_console_screen_scroll_up();
    } else 
    if ((new_pad & PSP_CTRL_DOWN) ||
        (new_pad & PSP_CTRL_CROSS)) {
      psp_console_put_key(DANZEFF_ENTER);
    } else
    if (new_pad & PSP_CTRL_UP) {
      psp_console_put_key('/');
    } else
    if (new_pad & PSP_CTRL_LEFT) {
      psp_console_put_key(DANZEFF_DEL);
    } else
    if (new_pad & PSP_CTRL_RIGHT) {
      psp_console_put_key(' ');
    }
  }
}

